// an example of using "3dlife.inc"
 

#include "colors.inc"

global_settings {
  assumed_gamma 1.0
}


// The array of cells will be centered on <0,0,0>
// and its size will be the array dimensions.
// For example, an 11x11x11 array will be as large
// as box{0,11}
camera {
  location  <-15.0, 12, 12>

  look_at   <0.0, 0.0,  0.0> 
  angle 40
}

sky_sphere {
  pigment {
    bozo
    color_map {
      [0.0 rgb <0.8,0.7,1.0>]
      [0.7 rgb <0.0,0.1,0.8>]
    }
    scale .5
  }
  
}

light_source {
  <-300, 100, 0>
  color rgb <1, 1, 1>
  
}


#include "3dlife.inc"

// declare these variables *after* calling 3dlife.inc
// so that they will override the defaults 

#declare RS1=seed(341);

// gbNewTexture is the texture given cells which came alive in the
// current generation
#declare gbNewTexture=texture{
        pigment{rgb<1,1,0>}
}                    

// gbOldTexture is the texture given live cells which were alive in the
// last generation
#declare gbOldTexture=texture{
        pigment{Red}
}      

// gbDiedTexture is the texture given cells which were alive in the
// last generation but are now dead.
#declare gbDiedTexture=texture{                

        pigment{White transmit 1} // invisible?
}


//#declare diffScale=0;// .5 makes 'old'  cells 50% larger than new ones.
                      // diffScale=0 keeps old and new the same size  

// Size of the cell array
// If these dimensions are odd numbers then there is a 'central cell'
// which can be at the center of a scene.
#declare xSize=7;
#declare ySize=7;
#declare zSize=7;

// Each of the three ways to get an initial generation requires some
// additional parameter
// 1)      To generate live cells randomly, livePercent needs to be specified. to generate live cells randomly
#declare livePercent=20; // livePercent=10 makes 10 percent of the array live cells                                           


// 2)     a file of cell values, initialGenerationFile, needs to be specified in order to make the
//      first generation from a file. The file needs to be the same format as those saved by the
//      3dlife macro 'writeCells(filename)
//#declare initialGenerationFile="glider5766.lif";
//      could use tempCellFileName to pickup from the end
//      of the last run
//#declare initialGenerationFile="lifecells.tmp";


// 3)     an array of indexes of the live cells needs to be specified in order
//      to generate from a 'live array'
//#declare liveArray=OOOArray; 
//      it might be convenient to define the liveArray about 0,0,0. 
//      For example:     
#declare liveArray=array[7][3]{{0,-1,0},{-1,0,0},{0,0,-1},{0,1,0},{1,0,0},{0,0,1},{0,0,0}} 
//      in which case, the cells need to be centered in the cell array (all indices must
//      be positive before computing the next generation).
#declare centerTheArray=1;


#declare myRules=array[4]{5,7,4,4}
//#declare myRules=rules5766
// The first two numbers in the rules array tell how many live neighbors a live
// cell must have in order to survive to the next generation. 
// For example, if you use a rule array 
//        array[4]{5,7,6,6}
// a live cell must have from 5 through 7 live neighbors in order to survive.
// The second pair (6,6 in the example) tell how many neighbors a dead (empty)
// cell needs in order to come alive. 5,7,6,6 says an empty cell must have
// at least 6 live neighbors and no fewer than 6 live neighbors to come alive


// a declared 'cellObject' replaces the default cube 
// cells are 1x1, and cellObjects should be centered on <0,0,0>
#declare cellObject=sphere{0,1/2}

// the scene dimensions of the entire cell array is xSize x ySize x zSize
object{
        
        generateSceneCells(xSize,ySize,zSize,myRules,fromRandom,cubeCells,torusWrap,clock) 

}
// ---- end of scene  

// The main macro to call is
// generateSceneCells(xS,yS,zS,rules,gen1From,cellType,wrap)
//
// xS = 'x' direction width of cell array (integer)

// yS = 'y' direction width of cell array (integer)

// zS = 'z' direction width of cell array (integer)

// rules = an array[4] of integers specifying the Life rules
// Using RulesArray=array[4]{r1,r2,r3,r4} 
// makes the rule: 
//        if a cell is already 'live' and it has from 
//        r1 through r2 (inclusive) live neighbors, then 
//                it remains alive in the next generation.
//                That is, it will survive. 
//        Otherwise it becomes 'dead'.
//        On the other hand. if the cell is now dead, then 
//        if it has from r3 through r4 (inclusive) live neighbors then 
//                it will be alive in the next generation.

// gen1From tells where the first generation comes from. 
//      fromLiveArray takes an array of cell indexes and each
//              cell in the array will be 'alive' in the first generation.
//              For example, using the array
//                      #declare liveArray= array[2][3]={{0,0,1},{0,0,0}}
//              would make two cells alive: the cell at 0,0,1 and the cell at 0,0,0
//              The second index of the array must be '3' since a cell is specified
//              by three indices
//   
//      fromFile gets the first generation from a file name by 'initialGenerationFile'
//              The file contains the cell Values for all of the cells in the Life game
//              cell value > 0 is alive. cell value <=0 is dead.
//              
//      fromRandom generates the first generation randomly with livePercent of the cells
//              alive.

// cellType is either 'cubeCells' (plain cubes occupy live cells) or 'blobCells' 
//      If 'diffScale' >0, then live cells which were 'just born' (weren't alive in the 
//      last generation) are green.
//      Alive cells which survived from the last generation are larger and red.
//      Cells which were alive in the last generation but are now dead are small and white

// wrap tells whether to wrap the cell array or not.
//      noWrap means that a cell on the edge of the cell array has no neighboring cells
//              outside the array.
//      torusWrap 'wraps' the array around from top to bottom and left to right. so a cell
//              at the edge has neighboring cells on the other side of the array.

// genNum (an integer) is just which generation this is. The first generation is '0'
//       Currently this is only important to the macro to distinguish the first generation 
//      from the rest